import cv2  # type: ignore
from Droplet_Detector.do_xslx import do_df  # type: ignore

sum_area_s = 0


def get_sum():
    return sum_area_s


def update_sum(new_value):
    global sum_area_s
    sum_area_s = new_value


def calculate_area(contours):
    # global sum_area_s
    all_area = 0

    for contour in contours:
        all_area += cv2.contourArea(contour)
    do_df(contours)
    update_sum(all_area)
    return all_area



"""
:authors: Kvas Andrey, Belkova Ksenia, Nekrasova Anna
:license: Apache License, Version 2.0, see LICENSE file

:copyright: (c) 2023 kvasik3000
"""

from .calculate import *  # noqa
from .get_contours import *  # noqa
from .Bot import *  # noqa
from .draw import *  # noqa
from .do_xslx import *  # noqa



import cv2  # type: ignore
import numpy as np
from Droplet_Detector.get_contours import get_contours  # type: ignore


def draw_contours_of_drops(img_path):
    # читаем пикчу и сохраняем копию

    img = cv2.imread(img_path)
    copy = img
    imgc = img.copy()
    print(type(imgc))

    img = cv2.medianBlur(img, 3)

    resize_times = 1.5
    img = cv2.resize(img, None, img, fx=1 / resize_times, fy=1 / resize_times)
    img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)

    # увеличиваем резкость изображения
    kernel = np.array([[-1, -1, -1], [-1, 9, -1], [-1, -1, -1]])
    img = cv2.filter2D(img, -1, kernel)

    # применяем маску собеля для выделения контуров
    sobel_f = cv2.Sobel(img, cv2.CV_64F, 2, 0, 7)
    abs_sobel_f = np.absolute(sobel_f)
    sobel = np.uint8(abs_sobel_f)
    print(type(sobel))

    # применяем размытие Гаусса для уменьшения шума
    img_blurred = (cv2.GaussianBlur(img, (2 * 2 + 1, 2 * 2 + 1), 0))

    # ещё ищем края
    canny = cv2.Canny(img_blurred, 40, 255,
                      L2gradient=True)
    # 50 - появляются шумы, но лучше отображаются контуры,
    # 100 - чуть меньше шума, чуть лучше контур

    # маску в чёрно-белый
    _, res = cv2.threshold(canny, 200, 255, cv2.THRESH_BINARY)

    # загружаем изображение и маску
    img = copy
    mask = cv2.convertScaleAbs(res)

    # проверка на размер изображений
    if img.shape[:2] != mask.shape[:2]:
        mask = cv2.resize(mask, img.shape[:2][::-1])

    # применяем маску к изображению
    masked_img = cv2.bitwise_and(img, img, mask=mask)
    print(type(masked_img))

    # рисуем маску на изображении
    contours, hierarchy = cv2.findContours(
        mask,
        cv2.RETR_EXTERNAL,
        cv2.CHAIN_APPROX_SIMPLE)

    cv2.fillPoly(mask, contours, (255, 255, 0))

    cn2 = get_contours(mask)

    cv2.drawContours(img, cn2, -1, (0, 255, 0), 1)

    filename = 'new_file/savedImage.jpg'
    cv2.imwrite(filename, img)
    return filename



import pandas as pd
import cv2  # type: ignore
from openpyxl import Workbook

num_of_table = 0


def get_num():
    return num_of_table


def update_num(new_value):
    global num_of_table
    num_of_table = new_value


def do_df(contours):
    len_contours = len(contours)
    df = pd.DataFrame()
    # Добавляем название столбца
    column_name = "Площадь капли"
    for i in range(len_contours):
        # Генерируем название строки
        row_name = f"Капля  {i + 1}"

        # Генерируем значение для столбца
        value = cv2.contourArea(contours[i])

        # Добавляем строку в датафрейм
        df.loc[row_name, column_name] = value
    to_exel(df)
    return df


def to_exel(data):
    global num_of_table
    workbook = Workbook()

    # Получаем активный лист
    sheet = workbook.active
    print(sheet)
    # Сохраняем рабочую книгу в файл
    workbook.save(f'exel/new{num_of_table}.xlsx')

    data.to_excel(f'exel/new{num_of_table}.xlsx')
    num_of_table += 1
    update_num(num_of_table)



import telebot  # type: ignore
from telebot import types  # type: ignore
from Droplet_Detector.draw import draw_contours_of_drops  # type: ignore
from Droplet_Detector.do_xslx import get_num  # type: ignore
from Droplet_Detector.calculate import get_sum  # type: ignore

# Создаем экземпляр бота
bot = telebot.TeleBot('6050349643:AAHh6MT-s12_9niKl_GwiYRt79NLXDvUk7A')
global_src = ' '


@bot.message_handler(commands=['start'])
def start(message):
    markup = types.ReplyKeyboardMarkup(resize_keyboard=True)
    btn1 = types.KeyboardButton('Меню 🗃')
    btn2 = types.KeyboardButton('Help❓')

    markup.row(btn1, btn2)
    bot.send_message(message.chat.id,
                     "Привет, {0.first_name}!".format(message.from_user),
                     reply_markup=markup)
    bot.send_message(message.chat.id,
                     "🔸Я бот🤖, который может:\n"
                     "🔸Отрисовать контур капель на фотографии💦\n"
                     "🔸Вычислить площадь каждой капли🧮\n"
                     "🔸Сохранить данные в xlsx файл и отправить его вам📝\n"
                     "Перед тем как начать работу нажмите Help❓ ",
                     reply_markup=markup)


@bot.message_handler(content_types=['document'])
def handle_docs_photo(message):
    try:

        file_down = bot.get_file(message.document.file_id)
        down_file = bot.download_file(file_down.file_path)

        src = 'save_docs/' + message.document.file_name
        with open(src, 'wb') as new_file:
            new_file.write(down_file)
        file = open(src, 'rb')
        # path = gray_img(src)
        # file_out = open(path, 'rb')

        global global_src
        global_src = src
        # print(global_src)

        bot.send_photo(message.chat.id, file)
        # os.remove(path)
        bot.reply_to(message, "Пожалуй, я это сохраню")

    except Exception as e:
        print(e)
        bot.reply_to(message, "Не получилось")


@bot.message_handler(content_types=['text'])
def one_click(message):
    if message.text == 'Меню 🗃':
        markup = types.ReplyKeyboardMarkup(resize_keyboard=True)
        item1 = types.KeyboardButton('Отрисовать контура капель')
        item2 = types.KeyboardButton('Рассчитать площадь капель')
        item3 = types.KeyboardButton('Сохранить в xlsx')
        item4 = types.KeyboardButton('⬅️ Назад')
        markup.add(item1, item2, item3, item4)
        bot.send_message(message.chat.id, "Меню 🗃", reply_markup=markup)
    elif message.text == '⬅️ Назад':
        markup = types.ReplyKeyboardMarkup(resize_keyboard=True)
        btn1 = types.KeyboardButton('Меню 🗃')
        btn2 = types.KeyboardButton('Help❓')
        markup.add(btn1, btn2)
        bot.send_message(message.chat.id, "🤔", reply_markup=markup)
    elif message.text == 'Рассчитать площадь капель':
        try:
            file_s = f'exel/new{get_num() - 1}.xlsx'
            file = open(file_s, 'rb')
            print(file)
            bot.send_message(message.chat.id,
                             f"Суммарная площадь капель: {get_sum()}\n"
                             f"Площадь каждой капли будет "
                             f"записана в табличку😉")

        except Exception as e:
            print(e)
            bot.send_message(message.chat.id,
                             "Извините, вы наверно не правильно поняли Help❓\n"
                             "Сперва вам нужно отрисовать контура капель!")
    elif message.text == 'Отрисовать контура капель':
        try:
            filename = draw_contours_of_drops(global_src)
            file = open(filename, 'rb')
            bot.send_photo(message.chat.id, file)
            bot.send_message(message.chat.id, "Отрисовал🎨")

        except Exception as e:
            print(e)
            bot.send_message(message.chat.id,
                             "Извините, но вы не можете продолжить работу.😔\n"
                             "Отправьте мне файл с фото и мы сможем начать!😉")

    elif message.text == 'Сохранить в xlsx':
        try:
            file_s = f'exel/new{get_num() - 1}.xlsx'
            file = open(file_s, 'rb')
            bot.send_document(message.chat.id, file)
            bot.send_message(message.chat.id, "Держи😇")

        except Exception as e:
            print(e)
            bot.send_message(message.chat.id,
                             "Извините, вы наверно не правильно поняли Help❓\n"
                             "Сперва вам нужно рассчитать площадь капель!")

    elif message.text == 'Help❓':
        markup = types.ReplyKeyboardMarkup(resize_keyboard=True)
        item = types.KeyboardButton('⬅️ Назад')
        markup.add(item)
        bot.send_message(message.chat.id,
                         "📌Для успешной работы бота отправьте"
                         " фотографию капель в виде файла\n"
                         "Далее нажмите <<Меню 🗃>>\n"
                         "📌Если вы хотите отрисовать контуры капель, нажмите"
                         " <<Отрисовать контура капель>> \n"
                         "📌Если вы хотите рассчитать площади капель, нажмите"
                         " <<Рассчитать площадь капель>>\n"
                         "📌Если вы хотите получить данные о "
                         "площадях в виде таблицы, нажмите"
                         " <<Сохранить в xlsx>>", reply_markup=markup)
        bot.send_message(message.chat.id,
                         "⁉Важно⁉\n"
                         "Соблюдайте последовательность:\n"
                         "<<Отрисовать контура капель>> ➡"
                         " <<Рассчитать площадь капель>> ➡ "
                         "<<Сохранить в xlsx>>", reply_markup=markup)
    else:
        bot.send_message(message.chat.id,
                         "Извините,мы не можем с вами "
                         "общаться на отдалённые темы, "
                         "прочтите пожалуйста Help❓ и узнайте"
                         " как со мной работать😉")


# Запускаем бота
bot.polling(none_stop=True, interval=0)



from io import open
from setuptools import setup  # type: ignore

"""
:authors: Kvas Andrey, Belkova Ksenia, Nekrasova Anna
:license: Apache License, Version 2.0, see LICENSE file
:copyright: (c) 2023 kvasik3000
"""

version = '0.1.3'

with open('README.md', encoding='utf-8') as f:
    long_description = f.read()

# with open("requirements.txt", "r") as file:
#     requirements = file.read().splitlines()

setup(
    name='Droplet_Detector',
    version=version,

    author='Kvas Andrey, Belkova Ksenia, Nekrasova Anna',
    author_email='superadrenoline3000@gmail.com',

    description='Telegram bot that draws the outlines of drips on a photo\n'
                '!!! Warning!!! \n'
                'Before running, be sure to create the folders'
                ' "exel", "new_file" and "save_docs" in the '
                'workspace where the python file is located !!!!',
    long_description=long_description,
    long_description_content_type='text/markdown',

    url='https://github.com/kvasik3000/Droplet-Detector',

    license='Apache License, Version 2.0, see LICENSE file',

    packages=['Droplet_Detector'],
    install_requires=[
        'numpy==1.24.3',
        'opencv-python==4.7.0.72',
        'openpyxl==3.1.2',
        'pandas==2.0.2',
        'pyTelegramBotAPI==4.11.0',
        'pytest==7.3.2',
        'telebot==0.0.5',
        'matplotlib==3.7.1',
        'pandas-stubs==2.0.2.230605',
        'openpyxl-stubs==0.1.25'
    ],

    classifiers=[
        "Programming Language :: Python :: 3.9",
        "Operating System :: OS Independent",
    ],
    python_requires='==3.9.*',
)
